@@ -60,8 +60,11 @@ extern FILE *popen();
 
 #include "pTk/tkPort.h"
 #include "pTk/Lang.h"
+#include "pTk/tkInt.h"
 #include "pTk/tkVMacro.h"
 
+#define VOID_SPRINTF 
+
 #ifdef __alpha
 #ifndef LONG64
 #define LONG64
