@@ -1,26 +1,25 @@
-# Copyright (c) 1995-1996 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-1997 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Wm;
-require Tk;
+use AutoLoader;
+
+require Tk::Widget;
+*AUTOLOAD = \&Tk::Widget::AUTOLOAD;
 
 use strict qw(vars);
 
 # There are issues with this stuff now we have Tix's wm release/capture
 # as toplevel-ness is now dynamic.
 
-BEGIN 
-{
- my $fn; 
- foreach $fn (qw(aspect client colormapwindows command deiconify focusmodel
-		frame geometry grid group iconbitmap iconify iconmask
-		iconname iconposition iconwindow maxsize minsize
-		overrideredirect positionfrom protocol resizable saveunder
-		sizefrom state title transient withdraw))
- {
-  *{"$fn"} = sub { shift->wm("$fn",@_) };
- }
-}
+use Tk::Submethods ( 'wm' => [qw(grid)] );
+
+Direct Tk::Submethods ('wm' => [qw(aspect client colormapwindows command 
+                       deiconify focusmodel frame geometry group
+                       iconbitmap iconify iconmask iconname iconposition
+                       iconwindow maxsize minsize overrideredirect positionfrom
+                       protocol resizable saveunder sizefrom state title transient
+                       withdraw)]);
 
 sub SetBindtags
 {
@@ -28,6 +27,19 @@ sub SetBindtags
  $obj->bindtags([ref($obj),$obj,'all']);
 }
 
+sub Populate
+{
+ my ($cw,$args) = @_;
+ $cw->ConfigSpecs('-overanchor' => ['PASSIVE',undef,undef,undef],
+                  '-popanchor'  => ['PASSIVE',undef,undef,undef],
+                  '-popover'    => ['PASSIVE',undef,undef,undef] 
+                 );
+}
+
+1;
+
+__END__
+
 sub Post
 {
  my ($w,$X,$Y) = @_;
@@ -47,15 +59,6 @@ sub AnchorAdjust
  return ($X,$Y);
 }
 
-sub Populate
-{
- my ($cw,$args) = @_;
- $cw->ConfigSpecs('-overanchor' => ['PASSIVE',undef,undef,undef],
-                  '-popanchor'  => ['PASSIVE',undef,undef,undef],
-                  '-popover'    => ['PASSIVE',undef,undef,undef] 
-                 );
-}
-
 sub Popup
 {
  my $w = shift;
@@ -92,7 +95,4 @@ sub Popup
  $w->Post($X,$Y);
 }
 
-1;
-__END__
-
 
